/*******************************************************************************
File:        TaskBarMsgWnd.cpp

Description: This file contains the module for creating a status bar message 
             window like MSN messenger

Created: Oct 13, 2001
Author:  Prateek Kaul
e-mail:  kaulpr@yahoo.com

Compiler with version number : Visual C++ 6.0

********************************************************************************/


/*********************************************************
	Original function comments removed from this file
	to make it easier to work with
	(more comments than code :P)
**********************************************************/

#include "stdafx.h"
#include "Resource.h"
#include "TaskBarMsgWnd.h"
#include "ContactView.h"
#include "Contact.h"
#include "Netmsg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(CTaskBarMsgWnd, CFrameWnd)
    ON_WM_SIZE()
    ON_WM_CREATE()
    ON_WM_MOUSEMOVE()
    ON_WM_TIMER()
    ON_WM_SETCURSOR()
    ON_WM_DESTROY()
    ON_WM_PAINT()
    ON_WM_LBUTTONDOWN()
    ON_MESSAGE(WM_MOUSELEAVE, OnMouseLeave)
    ON_MESSAGE(WM_MOUSEHOVER, OnMouseHover)
	ON_WM_CTLCOLOR()
END_MESSAGE_MAP()

CTaskBarMsgWnd::CTaskBarMsgWnd(
                      CString strMsg, 
                      unsigned int nWndWidth,
                      unsigned int nWndHeight,
                      unsigned int nMsgTimeOut,         
                      unsigned int nMsgWndCreationDelay, 
                      CRect rectMsgRect,
                      COLORREF clrBox,
                      COLORREF clrText,
				   CContactView *ContactView,
				   CContact *With,
				   CTaskBarMsgWnd *pLast //pass a pointer to the last one so we know where it was to stack it.

                  ) : m_strMsg(strMsg), m_rectMsgRect(rectMsgRect)
{
    m_nWndWidth  = nWndWidth;
    m_nWndHeight = nWndHeight;

    m_nMsgTimeOut          = nMsgTimeOut;
    m_nMsgWndCreationDelay = nMsgWndCreationDelay;

    m_bMouseOverWnd = FALSE;
	View = ContactView;
	Contact = With;
	Last = pLast;
    m_hCursor = NULL;

    m_clrBox  = clrBox;
    m_clrText = clrText;
	Top = 0;
	bmBack = (HBITMAP)::LoadImage(0, "frame.bmp", IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	icBack = (HICON)AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

CTaskBarMsgWnd::~CTaskBarMsgWnd()
{
//	::FreeResource(m_icBack);
}

void CTaskBarMsgWnd::PopWndForLeftTaskBar()
{
    // Create a specific new window class
    HCURSOR hCursor = ::LoadCursor(NULL, IDC_APPSTARTING);
    HBRUSH hBrush   = ::CreateSolidBrush(m_clrBox); 

	CString strWndClass = ::AfxRegisterWndClass(
                                CS_DBLCLKS,
                                hCursor,
                                hBrush
                            );

    CRect t_rect(0, 0, 0, 0);
    Create(strWndClass, NULL, WS_VISIBLE | WS_OVERLAPPEDWINDOW, t_rect);
    
    CRect rectDesktopWithoutTaskbar;   // The desktop area 

    // Get the desktop area
    ::SystemParametersInfo(SPI_GETWORKAREA, 0, &rectDesktopWithoutTaskbar, 0);
    
	//// Added - Matthew Millman ////
    // Calculate the actual width of the Window and its position
	if (Last)
	{
		m_nWndLeft   = rectDesktopWithoutTaskbar.left;
		m_nWndTop    = rectDesktopWithoutTaskbar.bottom - (rectDesktopWithoutTaskbar.bottom - Last->Top) - m_nWndHeight;
		m_nWndRight  = m_nWndLeft + m_nWndWidth;
		m_nWndBottom = m_nWndTop + m_nWndHeight;
	}
	else
	{
		m_nWndLeft   = rectDesktopWithoutTaskbar.left;
		m_nWndTop    = rectDesktopWithoutTaskbar.bottom - m_nWndHeight;
		m_nWndRight  = m_nWndLeft + m_nWndWidth;
		m_nWndBottom = m_nWndTop + m_nWndHeight;
	}
	
	Top = m_nWndTop;

	/////////////////////////////////

	m_nWndSize = 0; // The height of window is zero before showing

    SetTimer(IDT_POP_WINDOW_TIMER, m_nMsgWndCreationDelay, NULL);

}

void CTaskBarMsgWnd::PopWndForRightTaskBar()
{
    PopWndForBottomTaskBar();
}


void CTaskBarMsgWnd::PopWndForTopTaskBar()
{
    // Create a specific new window class
    HCURSOR hCursor = ::LoadCursor(NULL, IDC_APPSTARTING);
    HBRUSH hBrush   = ::CreateSolidBrush(m_clrBox); 

	CString strWndClass = ::AfxRegisterWndClass(
                                CS_DBLCLKS,
                                hCursor,
                                hBrush
                            );

    CRect t_rect(0, 0, 0, 0);
    Create(strWndClass, NULL, WS_VISIBLE | WS_OVERLAPPEDWINDOW, t_rect);
    
    CRect rectDesktopWithoutTaskbar;   // The desktop area 

    // Get the desktop area
    ::SystemParametersInfo(SPI_GETWORKAREA, 0, &rectDesktopWithoutTaskbar, 0);
    
	//// Added - Matthew Millman ////
	if (Last)
	{
		m_nWndLeft   = rectDesktopWithoutTaskbar.right - m_nWndWidth;
		m_nWndTop    = rectDesktopWithoutTaskbar.top + Last->Top - rectDesktopWithoutTaskbar.top;
		m_nWndRight  = m_nWndLeft + m_nWndWidth;
		m_nWndBottom = m_nWndTop + m_nWndHeight;
	}

	else
	{
    // Calculate the actual width of the Window and its position in screen co-ordinates
		m_nWndLeft   = rectDesktopWithoutTaskbar.right - m_nWndWidth;
		m_nWndTop    = rectDesktopWithoutTaskbar.top;
		m_nWndRight  = m_nWndLeft + m_nWndWidth;
		m_nWndBottom = m_nWndTop + m_nWndHeight;
	}

	Top = m_nWndBottom;

	/////////////////////////////////

    m_nWndSize = 0; // The height of window is zero before showing

    SetTimer(IDT_POP_WINDOW_TIMER, m_nMsgWndCreationDelay, NULL);
}

void CTaskBarMsgWnd::PopWndForBottomTaskBar()
{
    // Create a specific new window class
    HCURSOR hCursor = ::LoadCursor(NULL, IDC_APPSTARTING);
    HBRUSH hBrush   = ::CreateSolidBrush(m_clrBox); 

	CString strWndClass = ::AfxRegisterWndClass(
                                CS_DBLCLKS,
                                hCursor,
                                hBrush
                            );
    
    // Create the window with the registered class
    CRect t_rect(0, 0, 0, 0);
    Create(strWndClass, NULL, WS_EX_TOOLWINDOW | WS_VISIBLE | WS_OVERLAPPEDWINDOW, t_rect); 
    
    CRect rectDesktopWithoutTaskbar;   // The desktop area 

    // Get the desktop area
    ::SystemParametersInfo(SPI_GETWORKAREA, 0, &rectDesktopWithoutTaskbar, 0);
    
	//// Added - Matthew Millman ////

	if (Last)
	{
		m_nWndLeft   = rectDesktopWithoutTaskbar.right - m_nWndWidth;
		m_nWndTop    = rectDesktopWithoutTaskbar.bottom - (rectDesktopWithoutTaskbar.bottom - Last->Top) - m_nWndHeight;
		m_nWndRight  = m_nWndLeft + m_nWndWidth;
		m_nWndBottom = m_nWndTop + m_nWndHeight;
	}
	else
	{
    // Calculate the actual width of the Window and its position in screen co-ordinates
		m_nWndLeft   = rectDesktopWithoutTaskbar.right - m_nWndWidth;
		m_nWndTop    = rectDesktopWithoutTaskbar.bottom - m_nWndHeight;
		m_nWndRight  = m_nWndLeft + m_nWndWidth;
		m_nWndBottom = m_nWndTop + m_nWndHeight;
	}

    Top = m_nWndTop;

	/////////////////////////////////


    m_nWndSize = 0; // The height of window is zero before showing

    SetTimer(IDT_POP_WINDOW_TIMER, m_nMsgWndCreationDelay, NULL);
}

void CTaskBarMsgWnd::PopMsg()
{
    if (CheckIfTaskBarBottom())  // Most frequent case is status bar at bottom
    {
        PopWndForBottomTaskBar();
    }
    else
    {
        if (CheckIfTaskBarTop())
        {
            PopWndForTopTaskBar();
        }
        else
        {
            if (CheckIfTaskBarLeft())
            {
                PopWndForLeftTaskBar();
            }
            else
            {
                m_nTaskBarPos = STP_RIGHT; // Force it, as no need for calling "CheckIfTaskBarRight()
                PopWndForRightTaskBar();
            }
        }
    }
}

BOOL CTaskBarMsgWnd::CheckIfTaskBarLeft()
{
    unsigned int nAvailableScreenTop;
    unsigned int nAvailableScreenLeft;
    
    CRect rectDesktopWithoutTaskbar;   // The desktop area without status bar
    
    // Get the desktop area minus the status
    ::SystemParametersInfo(SPI_GETWORKAREA, 0, &rectDesktopWithoutTaskbar, 0);
    nAvailableScreenLeft  = rectDesktopWithoutTaskbar.left;
    nAvailableScreenTop = rectDesktopWithoutTaskbar.top;

    if ((nAvailableScreenLeft > 0) && (nAvailableScreenTop == 0))
    {
        m_nTaskBarPos = STP_LEFT;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

BOOL CTaskBarMsgWnd::CheckIfTaskBarRight()
{
    unsigned int nAvailableScreenWidth;
    unsigned int nAvailableScreenHeight;
    unsigned int nActualScreenWidth;
    unsigned int nActualScreenHeight;

    // Calculate the actual screen height and width
    nActualScreenWidth  = ::GetSystemMetrics(SM_CXFULLSCREEN);
    nActualScreenHeight = ::GetSystemMetrics(SM_CYFULLSCREEN);

  
    CRect rectDesktopWithoutTaskbar;   // The desktop area without status bar
    
    // Get the desktop area minus the status
    ::SystemParametersInfo(SPI_GETWORKAREA, 0, &rectDesktopWithoutTaskbar, 0);
    nAvailableScreenWidth  = rectDesktopWithoutTaskbar.Width();
    nAvailableScreenHeight = rectDesktopWithoutTaskbar.Height();

    if ((nAvailableScreenWidth != nActualScreenWidth) &&
        (nAvailableScreenHeight == nActualScreenHeight))
    {
        m_nTaskBarPos = STP_RIGHT;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

BOOL CTaskBarMsgWnd::CheckIfTaskBarTop()
{
    unsigned int nAvailableScreenTop;
    unsigned int nAvailableScreenLeft;
    
    CRect rectDesktopWithoutTaskbar;   // The desktop area without status bar
    
    // Get the desktop area minus the status
    ::SystemParametersInfo(SPI_GETWORKAREA, 0, &rectDesktopWithoutTaskbar, 0);
    nAvailableScreenLeft  = rectDesktopWithoutTaskbar.left;
    nAvailableScreenTop = rectDesktopWithoutTaskbar.top;

    if ((nAvailableScreenLeft == 0) && (nAvailableScreenTop > 0))
    {
        m_nTaskBarPos = STP_TOP;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

BOOL CTaskBarMsgWnd::CheckIfTaskBarBottom()
{
    unsigned int nAvailableScreenWidth;
    unsigned int nAvailableScreenBottom;
    unsigned int nActualScreenWidth;
    unsigned int nActualScreenBottom;

    // Calculate the actual screen height and width
    nActualScreenWidth  = ::GetSystemMetrics(SM_CXSCREEN);
    nActualScreenBottom = ::GetSystemMetrics(SM_CYSCREEN);

  
    CRect rectDesktopWithoutTaskbar;   // The desktop area without status bar
    
    // Get the desktop area minus the status
    ::SystemParametersInfo(SPI_GETWORKAREA, 0, &rectDesktopWithoutTaskbar, 0);
    nAvailableScreenWidth  = rectDesktopWithoutTaskbar.Width();
    nAvailableScreenBottom = rectDesktopWithoutTaskbar.bottom;

    if ((nAvailableScreenWidth == nActualScreenWidth) &&
        (nAvailableScreenBottom < nActualScreenBottom))
    {
        m_nTaskBarPos = STP_BOTTOM;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

void CTaskBarMsgWnd::OnSize(unsigned int nType, int cx, int cy)
{
    CFrameWnd::OnSize(nType, cx, cy);

    //SendMessage(WM_MOUSEMOVE, 0);
    CFont* pFont = NULL;
    CClientDC dc(this);

    if (m_bMouseOverWnd)
    {
        pFont = dc.SelectObject(&m_fontMessageUnderline);
    }
    else
    {
        pFont = dc.SelectObject(&m_fontMessageNoUnderline);
    }

    COLORREF clrPreviousBkColor = dc.SetBkColor(m_clrBox); 
    COLORREF clrPreviousTextColor = dc.SetTextColor(m_clrText);
    
    // Show with the message with the new font
	dc.SetBkMode(TRANSPARENT);
	dc.DrawState(CPoint(0, 0), CSize(130, 105), bmBack, NULL, NULL);
	dc.DrawIcon(CPoint(94, 68), icBack);
    dc.DrawText(m_strMsg, &m_rectMsgRect, DT_VCENTER | DT_CENTER | DT_WORDBREAK | DT_END_ELLIPSIS);

    dc.SelectObject(pFont);
    dc.SetBkColor(clrPreviousBkColor);
    dc.SetTextColor(clrPreviousTextColor);
}

int CTaskBarMsgWnd::OnCreate( LPCREATESTRUCT lpCreateStruct )
{
    if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	ModifyStyle(WS_CAPTION, 0, SWP_FRAMECHANGED); // removes title bar

    // Start creating two fonts, one underlined , other non underlined
    //
    // LOGFONT structure for font properties
    LOGFONT lf;
    ::ZeroMemory (&lf, sizeof (lf));
    lf.lfHeight = 100;
	lf.lfWeight = FW_NORMAL;
    lf.lfUnderline = TRUE;

    ::strcpy (lf.lfFaceName, _T("MS Sans Serif"));
	
    // Prepare for an underlined font
    m_fontMessageUnderline.CreatePointFontIndirect(&lf);

    // Prepare  an non undelined font
    lf.lfUnderline = FALSE;
    m_fontMessageNoUnderline.CreatePointFontIndirect(&lf);

    // Initialize the cursor.

#ifdef IDC_HAND //dang everyone complains about this
    m_hCursor = ::LoadCursor(NULL, IDC_HAND); // Use IDC_HAND if it compiles
#endif 
    return 0;
}

void CTaskBarMsgWnd::OnMouseMove(UINT nFlags, CPoint point)
{
    // Register the tracking of Mouse entering and leaveing the window
    // for WM_MOUSEHOVER and WM_MOUSELEAVE
    TRACKMOUSEEVENT t_MouseEvent;
    t_MouseEvent.cbSize      = sizeof(TRACKMOUSEEVENT);
    t_MouseEvent.dwFlags     = TME_LEAVE | TME_HOVER;
    t_MouseEvent.hwndTrack   = m_hWnd;
    t_MouseEvent.dwHoverTime = 1;

    ::_TrackMouseEvent(&t_MouseEvent);
}

void CTaskBarMsgWnd::OnTimer(UINT nIDEvent)
{
    switch (nIDEvent)
    {
        case IDT_POP_WINDOW_TIMER:    // When the window comes up
        {
            switch (m_nTaskBarPos)
            {
                case STP_BOTTOM:
                case STP_RIGHT:
                {
                    m_nWndSize += 2;
                    if (m_nWndSize > m_nWndHeight)
                    {
                        KillTimer(IDT_POP_WINDOW_TIMER);
                        SetTimer(IDT_SHOW_WINDOW_TIMER, m_nMsgTimeOut, NULL);
                    }
                    else
                    {
                        // Keep sizing the window, show it
                        SetWindowPos(
                            &wndTopMost,
                            m_nWndLeft,
                            m_nWndBottom - m_nWndSize, 
                            m_nWndWidth - 10,
                            m_nWndSize,
                            SWP_SHOWWINDOW
                        );
                    }
                }
                break;

                case STP_TOP:
                {
                    m_nWndSize += 2;
                    if (m_nWndSize > m_nWndHeight)
                    {
                        KillTimer(IDT_POP_WINDOW_TIMER);
                        SetTimer(IDT_SHOW_WINDOW_TIMER, m_nMsgTimeOut, NULL);
                    }
                    else
                    {
                        // Keep sizing the window, collapse it
                        SetWindowPos(
                            &wndTopMost,
                            m_nWndLeft,
                            m_nWndTop, 
                            m_nWndWidth - 10,
                            m_nWndSize,
                            SWP_SHOWWINDOW
                        );
                    }
                }
                break;

                case STP_LEFT:
                {
                    m_nWndSize += 2;
                    if (m_nWndSize > m_nWndHeight)
                    {
                        KillTimer(IDT_POP_WINDOW_TIMER);
                        SetTimer(IDT_SHOW_WINDOW_TIMER, m_nMsgTimeOut, NULL);
                    }
                    else
                    {
                        // Keep sizing the window, collpase it
                        SetWindowPos(
                            &wndTopMost,
                            m_nWndLeft + 10,
                            m_nWndBottom - m_nWndSize, 
                            m_nWndWidth - 10,
                            m_nWndSize,
                            SWP_SHOWWINDOW
                        );
                    }
                }
                break;
            }
        }
        break;

        case IDT_SHOW_WINDOW_TIMER:
        {
            KillTimer(IDT_SHOW_WINDOW_TIMER);
            SetTimer(IDT_COLLAPSE_WINDOW_TIMER, m_nMsgWndCreationDelay, NULL);
        }
        break;

        case IDT_COLLAPSE_WINDOW_TIMER:
        {
            switch (m_nTaskBarPos)
            {
                case STP_BOTTOM:
                case STP_RIGHT:
                {
                    m_nWndSize -= 2;
                    if (m_nWndSize <= 0) 
                    {
                        KillTimer(IDT_COLLAPSE_WINDOW_TIMER);
                        m_nWndSize = 0;

                        DestroyWindow();
						Quit(); // Use because it is in a separate thread
                    }
                    else
                    {
                        // Keep showing the window, collapse it
                        SetWindowPos(
                            &wndTopMost,
                            m_nWndLeft,
                            m_nWndBottom - m_nWndSize, 
                            m_nWndWidth - 10,
                            m_nWndSize,
                            SWP_SHOWWINDOW
                        );      
                    }
                }
                break;

                case STP_TOP:
                {
                    m_nWndSize -= 2;
                    if (m_nWndSize <= 0)
                    {
                        KillTimer(IDT_COLLAPSE_WINDOW_TIMER);
                        m_nWndSize = 0;

                        DestroyWindow();
						Quit(); // Use because it is in a separate thread
                    }
                    else
                    {
                        SetWindowPos(
                            &wndTopMost,
                            m_nWndLeft,
                            m_nWndTop, 
                            m_nWndWidth - 10,
                            m_nWndSize,
                            SWP_SHOWWINDOW
                        );
                    }
                }
                break;

                case STP_LEFT:
                {
                    m_nWndSize -= 2;
                    if (m_nWndSize <= 0)
                    {
                        KillTimer(IDT_COLLAPSE_WINDOW_TIMER);
                        m_nWndSize = 0;

                        DestroyWindow();
						Quit(); // Use because it is in a separate thread
                    }
                    else
                    {
                        SetWindowPos(
                            &wndTopMost,
                            m_nWndLeft + 10,
                            m_nWndBottom - m_nWndSize, 
                            m_nWndWidth - 10,
                            m_nWndSize,
                            SWP_SHOWWINDOW
                        );
                    }
                }
                break;
            }
        }
        break;
    }

}

LRESULT CTaskBarMsgWnd::OnMouseHover(WPARAM w, LPARAM l)
{
    if (m_bMouseOverWnd == FALSE) // Mouse was not on window
    {
        CClientDC dc(this);
        CFont* pFont = dc.SelectObject(&m_fontMessageUnderline);

        COLORREF clrPreviousColor = dc.SetBkColor(m_clrBox); 
        COLORREF clrPreviousTextColor = dc.SetTextColor(m_clrText);
        dc.SetBkMode(TRANSPARENT);
		dc.DrawState(CPoint(0, 0), CSize(130, 105), bmBack, NULL, NULL);
		dc.DrawIcon(CPoint(94, 68), icBack);
        dc.DrawText(m_strMsg, &m_rectMsgRect, DT_VCENTER | DT_CENTER | DT_WORDBREAK | DT_END_ELLIPSIS);
        dc.SelectObject(pFont);
        dc.SetBkColor(clrPreviousColor);
        dc.SetTextColor(clrPreviousTextColor);
    

        m_bMouseOverWnd = TRUE; // Mouse is now over window
    }
    return 0;
}

LRESULT CTaskBarMsgWnd::OnMouseLeave(WPARAM w, LPARAM l)
{
    if (m_bMouseOverWnd) // Mouse was over window, now it is leaving
    {
        CClientDC dc(this);
        CFont* pFont = dc.SelectObject(&m_fontMessageNoUnderline);

        COLORREF clrPreviousBkColor = dc.SetBkColor(m_clrBox); 
        COLORREF clrPreviousTextColor = dc.SetTextColor(m_clrText);
        dc.SetBkMode(TRANSPARENT);
		dc.DrawState(CPoint(0, 0), CSize(130, 105), bmBack, NULL, NULL);
		dc.DrawIcon(CPoint(94, 68), icBack);
        dc.DrawText(m_strMsg, &m_rectMsgRect, DT_VCENTER | DT_CENTER | DT_WORDBREAK | DT_END_ELLIPSIS);

        // Restore the DC to its previous state
        dc.SelectObject(pFont);
        dc.SetBkColor(clrPreviousBkColor);
        dc.SetTextColor(clrPreviousTextColor);

        m_bMouseOverWnd = FALSE; // Mouse is not over window
    }
    return 0;
}

BOOL CTaskBarMsgWnd::OnSetCursor(CWnd* pWnd , UINT nHitTest , UINT message)
{
    if (nHitTest == HTCLIENT)
    {
        ::SetCursor(m_hCursor);  // Set cursor to HAND type when mouse is over window

        return TRUE;
    }
    
    return CFrameWnd::OnSetCursor (pWnd, nHitTest, message);
}

void CTaskBarMsgWnd::OnDestroy(void)
{
    ::DeleteObject(m_hCursor); // Free the cursor as it is a shared resource..
	if (GetApp()->View->LastPopup)
	{
		switch (m_nTaskBarPos)
		{
		case STP_BOTTOM:
		case STP_RIGHT:
		case STP_LEFT:
			if (GetApp()->View->LastPopup->Top >= Top)
			GetApp()->View->LastPopup = NULL;
			break;
		case STP_TOP:
			if (Top >= GetApp()->View->LastPopup->Top)
			GetApp()->View->LastPopup = NULL;
			break;
		}
	}

    CFrameWnd::OnDestroy();
}

HBRUSH CTaskBarMsgWnd::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	return NULL;
}

void CTaskBarMsgWnd::OnPaint()
{
	CFont* pFont = NULL;
    CPaintDC dc(this);

    if (m_bMouseOverWnd)
    {
        pFont = dc.SelectObject(&m_fontMessageUnderline);
    }
    else
    {
        pFont = dc.SelectObject(&m_fontMessageNoUnderline);
    }

    COLORREF clrPreviousBkColor = dc.SetBkColor(m_clrBox); 
    COLORREF clrPreviousTextColor = dc.SetTextColor(m_clrText);
    
	// Show with the message with the new font
	dc.SetBkMode(TRANSPARENT);
	dc.DrawState(CPoint(0, 0), CSize(130, 105), bmBack, NULL, NULL);
	dc.DrawIcon(CPoint(94, 68), icBack);
    dc.DrawText(m_strMsg, &m_rectMsgRect, DT_VCENTER | DT_CENTER | DT_WORDBREAK | DT_END_ELLIPSIS);

    dc.SelectObject(pFont);
    dc.SetBkColor(clrPreviousBkColor);
    dc.SetTextColor(clrPreviousTextColor);
}

void CTaskBarMsgWnd::Quit()
{
	PostQuitMessage(0);
}

void CTaskBarMsgWnd::OnLButtonDown(UINT nFlags, CPoint point)
{
	View->NewConversation(Contact, NULL);
}

CTaskBarMsgWnd* CTaskBarMsgWnd::CreateObject(CString strMsg,
                                       unsigned int nWndWidth,
                                       unsigned int nWndHeight, 
                                       unsigned int nMsgTimeOut,         
                                       unsigned int nMsgWndCreationDelay, 
                                       CRect rectMsgRect,
                                       COLORREF clrBox,
                                       COLORREF clrText,
									   CContactView *ContactView,
									   CContact *With,
									   CTaskBarMsgWnd *pLast
                                    )
{
    CTaskBarMsgWnd* t_TaskBarMsgWnd = new CTaskBarMsgWnd(
												strMsg,
												nWndWidth,
												nWndHeight, 
												nMsgTimeOut,         
												nMsgWndCreationDelay, 
												rectMsgRect,
												clrBox,
												clrText,
												ContactView,
												With,
												pLast
                                              );


    return (t_TaskBarMsgWnd);
}

BOOL CTaskBarMsgWnd::PreCreateWindow(CREATESTRUCT& cs)
{
	if (!CFrameWnd::PreCreateWindow(cs))
    {
		return FALSE;
    }

 	// Create invisible parent window
 	if (!::IsWindow(m_wndInvisibleParent.m_hWnd))
 	{
        // Try creating the invisible parent window
        PCSTR pstrOwnerClass = ::AfxRegisterWndClass(0);
       
        BOOL bError = m_wndInvisibleParent.CreateEx(
                                    0,
                                    pstrOwnerClass, 
                                    _T(""), 
                                    WS_POPUP,
				                    CW_USEDEFAULT,
                                    CW_USEDEFAULT,
                                    CW_USEDEFAULT, 
                                    CW_USEDEFAULT,
				                    NULL,
                                    0
                                   );

        if (bError == FALSE)
        {
			return FALSE;
        }
 	}

    // Set the invisible window as the parent of "this" window
	cs.hwndParent = m_wndInvisibleParent.m_hWnd;

	return TRUE;
}
